@@ -196,8 +196,10 @@ Interface::OpenStatus InterfacePrivate::loadDeviceOptions()
     }
 
     // add extra option for selecting specific page sizes
-    BaseOption *pageSizeOption =
+    PageSizeOption *pageSizeOption =
         new PageSizeOption(optionTopLeftX, optionTopLeftY, optionBottomRightX, optionBottomRightY, optionResolution, optionPageWidth, optionPageHeight);
+    connect(this, &InterfacePrivate::optionsAboutToBeReloaded, pageSizeOption, &PageSizeOption::storeOptions);
+    connect(this, &InterfacePrivate::optionsReloaded, pageSizeOption, &PageSizeOption::restoreOptions);
     m_optionsList.append(pageSizeOption);
     m_externalOptionsList.append(new InternalOption(pageSizeOption));
     m_optionsLocation.insert(Interface::PageSizeOption, m_optionsList.size() - 1);
@@ -244,7 +246,7 @@ Interface::OpenStatus InterfacePrivate::loadDeviceOptions()
         connect(optionResolution, &BaseOption::valueChanged, m_scanThread, &ScanThread::setImageResolution);
     }
 
-    connect(m_scanThread, &ScanThread::scanProgressUpdated, q, &Interface::scanProgress);
+    connect(m_scanThread, &ScanThread::scanProgressUpdated, this, &InterfacePrivate::emitProgress);
     connect(m_scanThread, &ScanThread::finished, this, &InterfacePrivate::imageScanFinished);
 
     // try to set to default values
@@ -317,11 +319,13 @@ void InterfacePrivate::scheduleValuesReload()
 
 void InterfacePrivate::reloadOptions()
 {
+    Q_EMIT optionsAboutToBeReloaded();
     for (const auto option : std::as_const(m_optionsList)) {
         option->readOption();
         // Also read the values
         option->readValue();
     }
+    Q_EMIT optionsReloaded();
 }
 
 void InterfacePrivate::reloadValues()
@@ -331,6 +335,15 @@ void InterfacePrivate::reloadValues()
     }
 }
 
+void InterfacePrivate::emitProgress(int progress)
+{
+    if (m_previewScan) {
+        Q_EMIT q->previewProgress(progress);
+    } else {
+        Q_EMIT q->scanProgress(progress);
+    }
+}
+
 void InterfacePrivate::pollPollOptions()
 {
     for (int i = 1; i < m_optionsPollList.size(); ++i) {
@@ -340,39 +353,42 @@ void InterfacePrivate::pollPollOptions()
 
 void InterfacePrivate::imageScanFinished()
 {
-    Q_EMIT q->scanProgress(100);
+    emitProgress(100);
     if (m_scanThread->frameStatus() == ScanThread::ReadReady) {
-        Q_EMIT q->scannedImageReady(*m_scanThread->scanImage());
-        // now check if we should have automatic ADF batch scanning
-        if (m_executeMultiPageScanning && !m_cancelMultiPageScan) {
-            // in batch mode only one area can be scanned per page
-            Q_EMIT q->scanProgress(-1);
-            m_scanThread->start();
-            return;
-        }
-        // check if we should have timed batch scanning
-        if (m_batchMode->value().toBool() && !m_cancelMultiPageScan) {
-            // in batch mode only one area can be scanned per page
-            m_batchModeCounter = 0;
-            batchModeTimerUpdate();
-            m_batchModeTimer.start();
-            return;
-        }
-        // Check if we have a "wait for button" batch scanning
-        if (m_waitForExternalButton) {
-            qCDebug(KSANECORE_LOG) << "waiting for external button press to start next scan";
-            Q_EMIT q->scanProgress(-1);
-            m_scanThread->start();
-            return;
+        if (m_previewScan) {
+            Q_EMIT q->previewImageReady(*m_scanThread->scanImage());
+        } else {
+            Q_EMIT q->scannedImageReady(*m_scanThread->scanImage());
+            // now check if we should have automatic ADF batch scanning
+            if (m_executeMultiPageScanning && !m_cancelMultiPageScan) {
+                emitProgress(-1);
+                m_scanThread->start();
+                return;
+            }
+            // check if we should have timed batch scanning
+            if (m_batchMode->value().toBool() && !m_cancelMultiPageScan) {
+                // in batch mode only one area can be scanned per page
+                m_batchModeCounter = 0;
+                batchModeTimerUpdate();
+                m_batchModeTimer.start();
+                return;
+            }
+            // Check if we have a "wait for button" batch scanning
+            if (m_waitForExternalButton) {
+                qCDebug(KSANECORE_LOG) << "waiting for external button press to start next scan";
+                emitProgress(-1);
+                m_scanThread->start();
+                return;
+            }
         }
         scanIsFinished(Interface::NoError, QString());
     } else {
         switch (m_scanThread->saneStatus()) {
         case SANE_STATUS_GOOD:
-        case SANE_STATUS_CANCELLED:
-        case SANE_STATUS_EOF:
             scanIsFinished(Interface::NoError, sane_i18n(sane_strstatus(m_scanThread->saneStatus())));
             break;
+        case SANE_STATUS_CANCELLED:
+        case SANE_STATUS_EOF:
         case SANE_STATUS_NO_DOCS:
             Q_EMIT q->userMessage(Interface::Information, sane_i18n(sane_strstatus(m_scanThread->saneStatus())));
             scanIsFinished(Interface::Information, sane_i18n(sane_strstatus(m_scanThread->saneStatus())));
@@ -398,8 +414,50 @@ void InterfacePrivate::scanIsFinished(Interface::ScanStatus status, const QStrin
     if (m_optionsPollList.size() > 0 && !m_optionPollingNaughtylisted) {
         m_optionPollTimer.start();
     }
+    if (m_previewScan) {
+        // reset to user values for final scan
+        Option *topLeftXOption = q->getOption(Interface::TopLeftXOption);
+        Option *topLeftYOption = q->getOption(Interface::TopLeftYOption);
+        Option *bottomRightXOption = q->getOption(Interface::BottomRightXOption);
+        Option *bottomRightYOption = q->getOption(Interface::BottomRightYOption);
+        Option *bitDepthOption = q->getOption(Interface::BitDepthOption);
+        Option *previewOption = q->getOption(Interface::PreviewOption);
+        Option *resolutionOption = q->getOption(Interface::ResolutionOption);
+        Option *yResolutionOption = q->getOption(Interface::YResolutionOption);
+        Option *xResolutionOption = q->getOption(Interface::YResolutionOption);
 
-    Q_EMIT q->scanFinished(status, message);
+        if (topLeftXOption != nullptr) {
+            topLeftXOption->restoreSavedData();
+        }
+        if (topLeftYOption != nullptr) {
+            topLeftYOption->restoreSavedData();
+        }
+        if (bottomRightXOption != nullptr) {
+            bottomRightXOption->restoreSavedData();
+        }
+        if (bottomRightYOption != nullptr) {
+            bottomRightYOption->restoreSavedData();
+        }
+        if (resolutionOption != nullptr) {
+            resolutionOption->restoreSavedData();
+        }
+        if (yResolutionOption != nullptr) {
+            yResolutionOption->restoreSavedData();
+        }
+        if (xResolutionOption != nullptr) {
+            xResolutionOption->restoreSavedData();
+        }
+        if (bitDepthOption != nullptr) {
+            bitDepthOption->restoreSavedData();
+        }
+        if (previewOption != nullptr) {
+            previewOption->setValue(false);
+        }
+        m_previewScan = false;
+        Q_EMIT q->previewScanFinished(status, message);
+    } else {
+        Q_EMIT q->scanFinished(status, message);
+    }
 }
 
 void InterfacePrivate::determineMultiPageScanning(const QVariant &value)
