@@ -37,9 +37,13 @@ public:
     QString title() const override;
     QString description() const override;
     QVariantList valueList() const override;
+    QVariantList internalValueList() const override;
 
 public Q_SLOTS:
     bool setValue(const QVariant &value) override;
+    void storeOptions();
+    void restoreOptions();
+    void computePageSizes();
 
 private Q_SLOTS:
     void optionTopLeftXUpdated();
@@ -61,6 +65,7 @@ private:
     Option::OptionState m_state = Option::StateDisabled;
     QVariantList m_availableSizesListNames;
     QList<QSizeF> m_availableSizesList;
+    double m_previousCoordinates[4];
 };
 
 } // namespace KSaneCore
